@@ -15,9 +15,9 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: GtkNSSClientAuthDialogs.cpp 6589 2006-09-13 11:49:01Z chpe $
+ *  $Id: GtkNSSClientAuthDialogs.cpp 6952 2007-03-11 19:42:02Z chpe $
  */
 
 #include "mozilla-config.h"
@@ -55,6 +55,7 @@
 #include "ephy-debug.h"
 #include "ephy-gui.h"
 #include "ephy-state.h"
+#include "ephy-stock-icons.h"
 
 #include "AutoJSContextStack.h"
 #include "AutoWindowModalState.h"
@@ -170,7 +171,7 @@ GtkNSSClientAuthDialogs::ChooseCertificate (nsIInterfaceRequestor *ctx,
 					     GTK_WINDOW (dialog));
 	}
 
-	gtk_window_set_icon_name (GTK_WINDOW (dialog), "web-browser");
+	gtk_window_set_icon_name (GTK_WINDOW (dialog), EPHY_STOCK_EPHY);
 
 	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
 	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
