@@ -1,8 +1,8 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- *  Copyright (C) 2002  Ricardo Fernández Pascual
- *  Copyright (C) 2003, 2004  Marco Pesenti Gritti
- *  Copyright (C) 2003, 2004, 2005  Christian Persch
+ *  Copyright © 2002  Ricardo Fernández Pascual
+ *  Copyright © 2003, 2004  Marco Pesenti Gritti
+ *  Copyright © 2003, 2004, 2005  Christian Persch
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,9 +16,9 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: ephy-location-entry.c,v 1.79 2006/08/07 20:32:04 chpe Exp $
+ *  $Id: ephy-location-entry.c 7020 2007-05-01 17:13:10Z xan $
  */
 
 #include "config.h"
@@ -40,6 +40,7 @@
 #include <gtk/gtkentry.h>
 #include <gtk/gtkwindow.h>
 #include <gtk/gtkcellrenderertext.h>
+#include <gtk/gtkcellrendererpixbuf.h>
 #include <gtk/gtkcelllayout.h>
 #include <gtk/gtktreemodelsort.h>
 #include <gtk/gtkstock.h>
@@ -52,6 +53,7 @@
 #include <gtk/gtkseparatormenuitem.h>
 #include <gtk/gtkalignment.h>
 #include <gtk/gtkclipboard.h>
+#include <gtk/gtkversion.h>
 
 #include <string.h>
 
@@ -75,6 +77,9 @@ struct _EphyLocationEntryPrivate
 	guint action_col;
 	guint keywords_col;
 	guint relevance_col;
+	guint extra_col;
+	guint favicon_col;
+	guint url_col;
 
 	guint hash;
 
@@ -107,6 +112,7 @@ static const GtkTargetEntry url_drag_types [] =
 };
 
 static const GdkColor fallback_bg_colour = { 0, 0xf7f7, 0xf7f7, 0xbebe }; /* yellow-ish */
+static const GdkColor fallback_fg_colour = { 0, 0, 0, 0 }; /* black */
 
 static void ephy_location_entry_class_init (EphyLocationEntryClass *klass);
 static void ephy_location_entry_init (EphyLocationEntry *le);
@@ -135,7 +141,7 @@ ephy_location_entry_get_type (void)
 
 	if (G_UNLIKELY (type == 0))
 	{
-		static const GTypeInfo our_info =
+		const GTypeInfo our_info =
 		{
 			sizeof (EphyLocationEntryClass),
 			NULL,
@@ -208,8 +214,7 @@ ephy_location_entry_style_set (GtkWidget *widget,
 	}
 	else
 	{
-		/* FIXME: fall back to black text maybe? */
-		priv->secure_fg_colour = widget->style->text[GTK_STATE_NORMAL];
+		priv->secure_fg_colour = fallback_fg_colour;
 	}
 
 	if (bg_colour != NULL)
@@ -831,6 +836,33 @@ favicon_drag_data_get_cb (GtkWidget *widget,
 		time, entry, each_url_get_data_binder);
 }
 
+static gboolean
+icon_button_press_event_cb (GtkWidget *ebox,
+			    GdkEventButton *event,
+			    EphyLocationEntry *entry)
+{
+	guint state = event->state & gtk_accelerator_get_default_mod_mask ();
+
+	if (event->type == GDK_BUTTON_PRESS && 
+	    event->button == 1 && 
+	    state == 0 /* left */)
+	{
+		GtkWidget *toplevel;
+		EphyLocationEntryPrivate *priv = entry->priv;
+		
+		toplevel = gtk_widget_get_toplevel (GTK_WIDGET (entry));
+        	gtk_window_set_focus (GTK_WINDOW(toplevel),
+                              	      priv->icon_entry->entry);
+
+		gtk_editable_select_region (GTK_EDITABLE (priv->icon_entry->entry), 
+					    0, -1);
+
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
 static gboolean
 lock_button_press_event_cb (GtkWidget *ebox,
 			    GdkEventButton *event,
@@ -871,6 +903,8 @@ ephy_location_entry_construct_contents (EphyLocationEntry *entry)
 	gtk_drag_source_set (priv->icon_ebox, GDK_BUTTON1_MASK,
 			     url_drag_types, G_N_ELEMENTS (url_drag_types),
 			     GDK_ACTION_ASK | GDK_ACTION_COPY | GDK_ACTION_LINK);
+	g_signal_connect (priv->icon_ebox, "button-press-event",
+			  G_CALLBACK (icon_button_press_event_cb), entry);
 
 	gtk_tooltips_set_tip (priv->tips, priv->icon_ebox,
 			      _("Drag and drop this icon to create a link to this page"), NULL);
@@ -962,23 +996,52 @@ sort_func (GtkTreeModel *model,
 	return valueb - valuea;
 }
 
+#if GTK_CHECK_VERSION (2, 11, 0)
+static gboolean
+cursor_on_match_cb  (GtkEntryCompletion *completion,
+		     GtkTreeModel *model,
+		     GtkTreeIter *iter,
+		     EphyLocationEntry *le)
+{
+	char *item = NULL;
+	GtkWidget *entry;
+
+	gtk_tree_model_get (model, iter,
+			    le->priv->url_col,
+			    &item, -1);
+	
+	entry = gtk_entry_completion_get_entry (completion);
+	gtk_entry_set_text (GTK_ENTRY (entry), item);
+
+	g_free (item);
+
+	return TRUE;
+}
+#endif /* GTK+ 2.11.0 */
+
 void
 ephy_location_entry_set_completion (EphyLocationEntry *le,
 				    GtkTreeModel *model,
 				    guint text_col,
 				    guint action_col,
 				    guint keywords_col,
-				    guint relevance_col)
+				    guint relevance_col,
+				    guint extra_col,
+				    guint favicon_col,
+				    guint url_col)
 {
 	EphyLocationEntryPrivate *priv = le->priv;
 	GtkTreeModel *sort_model;
 	GtkEntryCompletion *completion;
-	GtkCellRenderer *cell;
+	GtkCellRenderer *cell, *extracell, *iconcell;
 
 	le->priv->text_col = text_col;
 	le->priv->action_col = action_col;
 	le->priv->keywords_col = keywords_col;
 	le->priv->relevance_col = relevance_col;
+	le->priv->extra_col = extra_col;
+	le->priv->favicon_col = favicon_col;
+	le->priv->url_col = url_col;
 
 	sort_model = gtk_tree_model_sort_new_with_model (model);
 	g_object_unref (model);
@@ -995,12 +1058,43 @@ ephy_location_entry_set_completion (EphyLocationEntry *le,
 	g_signal_connect_after (completion, "action-activated",
 				G_CALLBACK (action_activated_after_cb), le);
 
+	iconcell = gtk_cell_renderer_pixbuf_new ();
+
+	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (completion),
+				    iconcell, FALSE);
+	gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (completion),
+				       iconcell, "pixbuf", favicon_col);
+
 	cell = gtk_cell_renderer_text_new ();
+	g_object_set (cell, 
+			"ellipsize", PANGO_ELLIPSIZE_END,
+			"ellipsize-set", TRUE,
+			NULL);
 	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (completion),
 				    cell, TRUE);
 	gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (completion),
 				       cell, "text", text_col);
 
+	extracell = gtk_cell_renderer_text_new ();
+	g_object_set (extracell, 
+			"ellipsize", PANGO_ELLIPSIZE_END,
+			"ellipsize-set", TRUE,
+			"foreground", "Gray",
+			"foreground-set", TRUE,
+			"alignment", PANGO_ALIGN_RIGHT,
+			NULL);
+
+	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (completion),
+				    extracell, TRUE);
+	gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (completion),
+				       extracell, "text", extra_col);
+
+#if GTK_CHECK_VERSION (2, 11, 0)
+	g_object_set (completion, "inline-selection", TRUE, NULL);
+	g_signal_connect (completion, "cursor-on-match",
+			  G_CALLBACK (cursor_on_match_cb), le);
+#endif
+
 	gtk_entry_set_completion (GTK_ENTRY (priv->icon_entry->entry), completion);
 	g_object_unref (completion);
 }
