@@ -1,7 +1,7 @@
 /*
- *  Copyright (C) 2002  Ricardo Fernández Pascual
- *  Copyright (C) 2003, 2004  Marco Pesenti Gritti
- *  Copyright (C) 2003, 2004, 2005  Christian Persch
+ *  Copyright © 2002  Ricardo Fernández Pascual
+ *  Copyright © 2003, 2004  Marco Pesenti Gritti
+ *  Copyright © 2003, 2004, 2005  Christian Persch
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,9 +15,9 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: ephy-location-entry.h,v 1.19 2005/04/10 19:28:10 chpe Exp $
+ *  $Id: ephy-location-entry.h 7020 2007-05-01 17:13:10Z xan $
  */
 
 #ifndef EPHY_LOCATION_ENTRY_H
@@ -47,11 +47,11 @@ struct _EphyLocationEntryClass
 	GtkToolItemClass parent_class;
 
 	/* Signals */
-	void	(*user_changed)	  (EphyLocationEntry *entry);
-	void	(* lock_clicked)  (EphyLocationEntry *entry);
+	void   (* user_changed)	(EphyLocationEntry *entry);
+	void   (* lock_clicked)	(EphyLocationEntry *entry);
 	/* for getting the drag data */
-	char *	(* get_location)  (EphyLocationEntry *entry);
-	char *	(* get_title)     (EphyLocationEntry *entry);
+	char * (* get_location)	(EphyLocationEntry *entry);
+	char * (* get_title)	(EphyLocationEntry *entry);
 };
 
 struct _EphyLocationEntry
@@ -71,14 +71,18 @@ void		ephy_location_entry_set_completion	(EphyLocationEntry *le,
 							 guint text_col,
 							 guint action_col,
 							 guint keywords_col,
-							 guint relevance_col);
+							 guint relevance_col,
+							 guint extra_col,
+							 guint favicon_col,
+							 guint url_col);
 
 void		ephy_location_entry_set_location	(EphyLocationEntry *le,
-							 const char *new_location);
+							 const char *address,
+							 const char *typed_address);
 
 const char     *ephy_location_entry_get_location	(EphyLocationEntry *le);
 
-void		ephy_location_entry_restore_location	(EphyLocationEntry *le);
+gboolean	ephy_location_entry_reset		(EphyLocationEntry *entry);
 
 void		ephy_location_entry_activate		(EphyLocationEntry *le);
 
@@ -87,6 +91,9 @@ GtkWidget      *ephy_location_entry_get_entry		(EphyLocationEntry *entry);
 void		ephy_location_entry_set_favicon		(EphyLocationEntry *entry,
 							 GdkPixbuf *pixbuf);
 
+void		ephy_location_entry_set_secure		(EphyLocationEntry *entry,
+							 gboolean secure);
+
 void		ephy_location_entry_set_show_lock	(EphyLocationEntry *entry,
 							 gboolean show_lock);
 
